@@ -9,7 +9,7 @@ from typing import List, Optional
 
 from pydantic import BaseModel, Field
 
-from lunchable.config import APIConfig
+from lunchable._config import APIConfig
 from lunchable.models._core import LunchMoneyAPIClient
 
 logger = logging.getLogger(__name__)
@@ -23,7 +23,7 @@ class TagsObject(BaseModel):
     """
 
     id: int = Field(description="Unique identifier for tag")
-    name: str = Field(description="User-defined name of tag")
+    name: str = Field(description="User-defined name of tag", min_length=1)
     description: Optional[str] = Field(description="User-defined description of tag")
 
 
@@ -45,7 +45,7 @@ class _LunchMoneyTags(LunchMoneyAPIClient):
         -------
         List[TagsObject]
         """
-        response_data = self._make_request(method=self.methods.GET,
+        response_data = self._make_request(method=self.Methods.GET,
                                            url_path=APIConfig.LUNCHMONEY_TAGS)
         tag_objects = [TagsObject(**item) for item in response_data]
         return tag_objects
